/*******************************************************************************

    COFRINHO DE COMBINAO USANDO POTNCIOMETRO LINEAR DE 10K
    
    (MTODO DE INSERO POR TEMPO PARADO NO POTENCIMETRO)


    C/ PIC12F675


    AUTOR: CLAUDIO LARIOS

    DATA:17/12/2023

    SUGESTO: USE KNOB COM ESCALA NUMERADA PARA MELHOR OPERAO.


    MODO PROGRAMAO: MANTENHA BOTO 'PROG' APERTADO E LIGUE A
         ALIMENTAO. LED 'PROG' ACENDER.
         ENTRE COM A SENHA:
         POSICIONANDO O KNOB INICIALMENTE NA POSIO ZERO PARA COMEAR
         A INSERIR A SENHA. VIRE O KNOB AT A POSIO DESEJADA.
         AGUARDE PARADO O LED PISCAR PARA INDICAR A INSERO.
         GIRE O KNOB AT A PRXIMA POSIO DESEJADA E AGUARDE SEM MOVER
         AT QUE O LED PISQUE.
         REPITA O PROCESSO COM TODA A SENHA. AO ENTRAR A LTIMA POSIO
         O LED 'PROG' APAGAR, INDICANDO SUCESSO NA GRAVAO.

    USO: POSICIONE O KNOB INICIALMENTE EM ZERO PARA INSERIR A SENHA.
    O LED ACENDE. COLOQUE A SENHA DE 5 DGITOS.
    OBS. O TEMPO PARADO AO GIRAR O KNOB DO POTENCIMETRO  USADO PARA
    SINALIZAR A INSERO DO DGITO. MOVER MUITO LENTAMENTE O KNOB DO
    POTENCIMETRO PODER DISPARAR ESTE MECANISMO DE INSERO.

    OBS. SEMPRE COLOQUE O POTENCIMETRO NA POSIO ZERO QUANDO FOR ENTRAR
    COM UMA SENHA. O LED ACENDER PARA INDICAR O NICIO DA INSERO DE
    SENHA. CASO ERRE A INSERO DA SENHA, ESPERE O LED APAGAR APS UNS
    20 SEGUNDOS E REPITA O PROCESSO DE INSERO DA SENHA.


    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, MCLR, PUT, BROWNOUT

//bits
#bit bprog    =  0x05.5 // pino 2
#bit ledprog  =  0x05.0 // pino 7
#bit led      =  0x05.1 // pino 6
#bit out      =  0x05.4 // pino 3

//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  wpu    = 0x95
#byte  option_reg = 0x81

#define TEMPO_PARADA  100
#define TEMPO_ENTRAR_SENHA 500 // tempo para entrar a senha depois de acionado boto
#define TEMPO_PROG_MAX 300000 // tempo de time out para a programao
#define QUANT_DIGITOS  5  //quantidade de digitos na senha = 5
#define TEMPO_ACIONAMENTO 5000 //tempo que fechadura eltrica  acionada (em ms)
//==============================================================================

//variveis globais
unsigned int1 btn=1,flagat=0;
unsigned int8 val[QUANT_DIGITOS];
unsigned int8 g,h,ptn1=0;
unsigned int16 timer7=0;
unsigned int32 timer5=0,timer6=0;

/*******************************************************************************
          SUB-ROTINAS
*******************************************************************************/
/*******************************************************************************
          le_adc()
*******************************************************************************/
//rotina de leitura do potencimetro e acondicionamento de valores
//faz 16 leituras sucessivas e divide por 16 (adc em 8 bits)
unsigned int8 le_adc(){
  unsigned int8 a,b;
  unsigned int16 soma;

   soma=0;

   for(a=0;a<16;a++){
     soma= soma + READ_ADC();
   }

   b=soma/16;//obtm mdia
   b=b/10;//obtem apenas 25 valores diferentes

   
   return(b);//retorna valor
}

/*******************************************************************************
          prog_mode()
*******************************************************************************/
//Modo de programao de senha pelo potencimetro e boto
void prog_mode(){

  if(!bprog){
    delay_ms(50);
    if(!bprog){
      ledprog=1;//acende led
      timer5=TEMPO_PROG_MAX;
    }
  }
}
/*******************************************************************************
          time_out_prog()
*******************************************************************************/
//sai do modo de programao por tempo
void time_out_prog(void){
  if(ledprog){
   timer5--;
   if(timer5==0)ledprog=0;
  }
}
/*******************************************************************************
          time_out_abert()
*******************************************************************************/
//sai do modo de insero de senha por tempo
void time_out_insercao(void){
  if(led){
   timer6--;
   if(timer6==0){led=0; ptn1=0;delay_ms(1000);     timer6=1;}//xxxxx
  }
}
/*******************************************************************************
          time_out_parado()
*******************************************************************************/
//aguarda tempo sem movimento do pot para inserir senha
void time_out_parado(void){
   if(timer7==0)return;
   if(flagat==0)return;
   timer7--;
   if(timer7==0){
     
     led=0;
     delay_ms(200);
     led=1;
     btn=0;
     flagat=0;
   }//if(timer7==0){
}


/*******************************************************************************
          insercao()
*******************************************************************************/
//verifica nicio da insero de senha
void inicio_insercao(void){
 if(led)return;
 if(READ_ADC()==0){//verifica se pot foi levado a zero
   led=1;// inicia a insero
   timer6= TEMPO_ENTRAR_SENHA;//carrega tempo mximo em modo de insero
   ptn1=0;
   flagat=0;//despreza leituras iniciais
 }
}
/*******************************************************************************
          inserindo_senha()
*******************************************************************************/
//rotina de insero de senha
void inserindo_senha(void){
  unsigned int8 b,c;
  if(!led){ flagat=0; return;}

  b= le_adc();
  delay_ms(50);//20
  c= le_adc();
 
  if((b == c) && (flagat==0)){timer7=TEMPO_PARADA; return;}// medies iniciais em '0'

  if(c > b){flagat=1; return;}//timer7=TEMPO_PARADA;
  if(b > c){flagat=1; return;}
  if(b == c)time_out_parado();//decrementa contador e ao final ativa btn=0



}

/*******************************************************************************
          ROTINA MAIN
*******************************************************************************/

void main() {

   setup_adc_ports(sAN2 | VSS_VDD);  //seleciona AN2(pino 5) e 5v como referncia
   setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   set_adc_channel(2); //ajusta para o pino 5 (AN2)
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores
   setup_vref(FALSE);//desliga vref
   trisio=0b101100;//ajusta entradas e sadas
   gpio=0;//zera tudo
   wpu=0b100000;//liga pull up individual pino 2
   option_reg=0x80; //liga pull up
   prog_mode();//testa boto para modo de programao



/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


   for(;;){

     inicio_insercao();//verifica insero de senha
     inserindo_senha();//rotina de insero de senha


     if(btn==0){ //testa boto de insero de senha
      timer5=TEMPO_PROG_MAX; //reseta tempo de programao
      timer6= TEMPO_ENTRAR_SENHA;//inicia tempo de time out
      val[ptn1]=le_adc();//le valor da posio atual do potencimetro e salva no array
      h=ptn1;//pega posio atual do ponteiro
      ptn1++;//incrementa ponteiro do array
      if(ledprog==1){//se em modo de programao ento grava digitos na eeprom
        write_eeprom(h,val[h]);//salva na eeprom
        if(ptn1>=QUANT_DIGITOS){//verifica se foi o ltimo nmero da senha
        ledprog=0;//apaga led de programao
        led=0;//apaga led de inicio de insero de senha
        ptn1=0;//zera ponteiro do array
        }
      }
      else{//se no est em programao est ento em modo de execuo
        if(ptn1>=QUANT_DIGITOS){
          for(g=0;g<QUANT_DIGITOS;g++){//compara valor inserido com a eeprom
            if(val[g]==read_eeprom(g))continue;//se igual continua no loop
            ptn1=0;//senha errada, reseta ponteiro das senhas e
            led=0;//apaga led
            break;//sai do loop forado
          }//for
          if(g>=QUANT_DIGITOS) {
              led=0;
              out=1;//liga fechadura eltrica
              delay_ms(TEMPO_ACIONAMENTO);//da um tempo acionando a fechadura eltrica
              out=0;//desliga fechadura
              ptn1=0;//reseta ponteiro das senhas
            }
        }
      }//else

     // timer7=TEMPO_PARADA;
      btn=1;
      flagat=0;
     }// if(btn==0)



   time_out_parado();//tempo sem movimento no pot
   time_out_insercao();//conta tempo mximo para inserir senha
   time_out_prog();//conta tempo mximo para programao
   }//for(;;)

}//void main()
