/*******************************************************************************

    COFRINHO DE COMBINAO USANDO POTNCIOMETRO LINEAR DE 10K E UM BOTO


    C/ PIC12F675


    AUTOR: CLAUDIO LARIOS

    DATA:17/12/2023

    SUGESTO: USE KNOB COM ESCALA NUMERADA PARA MELHOR OPERAO.

    USO: POSICIONE O KNOB NO PRIMEIRO NMERO DA COMBINAO E
         APERTE O BOTO. REPITA COM OS OUTROS NMEROS.
    MODO PROGRAMAO: MANTENHA BOTO 'PROG' APERTADO E LIGUE A
         ALIMENTAO. LED 'PROG' ACENDER. ENTRE COM A SENHA,
         POSICIONANDO O KNOB NA POSIO DESEJADA E APERTE O BOTAO
         PARA GRAVAR. IR PISCAR LED 'PROG'. REPITA COM OS OUTROS
         NMEROS. AO ENTRAR O LTIMO NMERO O LED 'PROG' APAGAR.

    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS

*******************************************************************************/
#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT

//bits
#bit bprog    =  0x05.5 // pino 2
#bit ledprog  =  0x05.0 // pino 7
#bit led      =  0x05.1 // pino 7
#bit out      =  0x05.4 // pino 3
#bit btn      =  0x05.3 // pino 4
//bytes
#byte  trisio = 0x85
#byte  gpio   = 0x05
#byte  wpu    = 0x95
#byte  option_reg = 0x81


#define TEMPO_ENTRAR_SENHA 200000 // tempo para entrar a senha depois de acionado boto
#define TEMPO_PROG_MAX 500000 // tempo de time out para a programao
#define QUANT_DIGITOS  5  //quantidade de digitos na senha = 5
#define TEMPO_ACIONAMENTO 5000 //tempo que fechadura eltrica  acionada (em ms)
//==============================================================================

//variveis globais

unsigned int8 val[5];
unsigned int8 g,h,ptn1=0;
unsigned int32 timer5=0,timer6=0;

/*******************************************************************************
          SUB-ROTINAS
*******************************************************************************/
/*******************************************************************************
          le_adc()
*******************************************************************************/
//rotina de leitura do potencimetro e acondicionamento de valores
//faz 16 leituras sucessivas e divide por 16 (adc em 8 bits)
unsigned int8 le_adc(){
  unsigned int8 a,b;
  unsigned int16 soma;
     
   soma=0;
     
   for(a=0;a<16;a++){
     soma= soma + READ_ADC();
   }
   
   b=soma/16;//obtm mdia
   b=b/25;//obtem apenas 10 valores diferentes
   return(b);//retorna valor
}
   
/*******************************************************************************
          prog_mode()
*******************************************************************************/
//Modo de programao de senha pelo potencimetro e boto
void prog_mode(){

  if(!bprog){
    delay_ms(50);
    if(!bprog){
      ledprog=1;//acende led
      timer5=TEMPO_PROG_MAX;
    }
  }
}
/*******************************************************************************
          time_out_prog()
*******************************************************************************/
//sai do modo de programao por tempo
void time_out_prog(void){
  if(ledprog){
   timer5--;
   if(timer5==0)ledprog=0;
  }
}
/*******************************************************************************
          time_out_abert()
*******************************************************************************/
//sai do modo de insero de senha por tempo
void time_out_abert(void){
  if(led){
   timer6--;
   if(timer6==0){led=0; ptn1=0;}
  }
/*******************************************************************************
          timer6_init()
*******************************************************************************/
//inicializa timer de tempo de insero da senha
}
void timer6_init(void){
  timer6= TEMPO_ENTRAR_SENHA;
  led=1;//indica inicio de entrada de senha
}


/*******************************************************************************
          ROTINA MAIN
*******************************************************************************/

void main() {

   setup_adc_ports(sAN2 | VSS_VDD);  //seleciona AN2(pino 5) e 5v como referncia
   setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   set_adc_channel(2); //ajusta para o pino 5 (AN2)
   setup_counters(RTCC_INTERNAL,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);//desliga timer 1
   setup_comparator(NC_NC_NC_NC);//desliga comparadores
   setup_vref(FALSE);//desliga vref
   trisio=0b101100;//ajusta entradas e sadas
   gpio=0;//zera tudo
   wpu=0b100000;//liga pull up individual pino 2
   option_reg=0x80; //liga pull up
   prog_mode();//testa boto para modo de programao

   

/*******************************************************************************
           LOOP PRINCIPAL DE REPETIO
*******************************************************************************/


   for(;;){   
     
     if(btn==0){ //testa boto de insero de senha
      timer5=TEMPO_PROG_MAX; //reseta tempo de programao   
      timer6_init();//inicia tempo de time out
      val[ptn1]=le_adc();//le valor da posio atual do potencimetro e salva no array
      h=ptn1;//pega posio atual do ponteiro
      ptn1++;//incrementa ponteiro do array
      if(ledprog==1){//se em modo de programao ento grava digitos na eeprom
        write_eeprom(h,val[h]);//salva na eeprom
        if(ptn1>=QUANT_DIGITOS){//verifica se foi o ltimo nmero da senha
        ledprog=0;//apaga led de programao
        led=0;//apaga led de inicio de insero de senha
        ptn1=0;//zera ponteiro do array
        }
      }
      else{//se no est em programao est ento em modo de execuo
        if(ptn1>=QUANT_DIGITOS){
          for(g=0;g<QUANT_DIGITOS;g++){//compara valor inserido com a eeprom
            if(val[g]==read_eeprom(g))continue;//se igual continua no loop
            ptn1=0;//senha errada, reseta ponteiro das senhas e
            led=0;//apaga led
            break;//sai do loop forado           
          }//for
          if(g>=QUANT_DIGITOS) {
              led=0;
              out=1;//liga fechadura eltrica
              delay_ms(TEMPO_ACIONAMENTO);//da um tempo acionando a fechadura eltrica
              out=0;//desliga fechadura
              ptn1=0;//reseta ponteiro das senhas
            }
        }
      }//else
      
      
      while(btn==0)delay_ms(40);//aguarda soltar boto de entrada de senha
     }// if(btn==0)
   



   time_out_abert();//conta tempo mximo para abertura
   time_out_prog();//conta tempo mximo para programao
   }//for(;;)

}//void main()
